@@ -9,8 +9,7 @@ class CRFConfig:
         num_iter: int = 100,
         test_size: int = 0.3,
         verbose: bool = False,
-        c1: float = 1.0,
-        c2: float = 1.0,
+        c_values: list = [0.01, 0.1, 0.5],
         bias: bool = True,
         window_size: int = 2,
         suffix_len: tuple = (2,2),
@@ -24,8 +23,7 @@ class CRFConfig:
         self.verbose = verbose
 
         self.num_iter = num_iter
-        self.c1 = c1
-        self.c2 = c2
+        self.c_values = c_values
 
         self.feature_layers = feature_layers
         self.context_feature_layers = context_feature_layers
