@@ -37,6 +37,7 @@ class CRFExtractor:
         self.config = config
         self.embedding = embedding
         self.model = None
+        self.best_c_values = None
 
 
     def __str__(self):
@@ -139,30 +140,51 @@ class CRFExtractor:
         y_train = [sent2labels(s) for s in train_documents]
         X_test = [sent2features(s, self.config, self.embedding) for s in test_documents]
         y_test = [sent2labels(s) for s in test_documents]
-        # create trainer
-        trainer = pycrfsuite.Trainer(verbose=self.config.verbose)
-        # feed data to trainer
-        for xseq, yseq in zip(X_train, y_train):
-            trainer.append(xseq, yseq)
-        # set trainer params
-        trainer.set_params({
-            'c1': self.config.c1, # coefficient for L1 penalty
-            'c2': self.config.c2, # coefficient for L2 penalty
-            'max_iterations': self.config.num_iter, # stop earlier
-            'feature.possible_transitions': True # include transitions that are possible, but not observed
-        })
-        # train & save the model
-        trainer.train(save_path)
-        # load tagger model for validation
-        tagger = pycrfsuite.Tagger()
-        tagger.open(save_path)
-        # evaluate model
-        y_pred = [tagger.tag(xseq) for xseq in X_test]
-        report = TaggingReport(y_test, y_pred, self.config.labels)
+
+
+        
+        # Test different C-values to get optimal model.
+        best_f1 = 0
+        best_report = None
+        # Test all combinations
+        for c1_value in self.config.c_values:
+            for c2_value in self.config.c_values:
+                # create trainer
+                trainer = pycrfsuite.Trainer(verbose=self.config.verbose)
+                # feed data to trainer
+                for xseq, yseq in zip(X_train, y_train):
+                    trainer.append(xseq, yseq)
+                # set trainer params
+                trainer.set_params({
+                    'c1': c1_value, # coefficient for L1 penalty
+                    'c2': c2_value, # coefficient for L2 penalty
+                    'max_iterations': self.config.num_iter, # stop earlier
+                    'feature.possible_transitions': True # include transitions that are possible, but not observed
+                })
+                # create temp path for the temp model
+                temp_save_path = f"{save_path}_{uuid.uuid4()}"
+                # train & save the model
+                trainer.train(temp_save_path)
+                # load tagger model for validation
+                tagger = pycrfsuite.Tagger()
+                tagger.open(temp_save_path)
+                # evaluate model
+                y_pred = [tagger.tag(xseq) for xseq in X_test]
+                report = TaggingReport(y_test, y_pred, self.config.labels)
+                # check if model better than our best
+                if report.f1_score > best_f1:
+                    # found a new best model!
+                    best_f1 = report.f1_score
+                    self.model = tagger
+                    best_report = report
+                    self.best_c_values = [c1_value, c2_value]
+                    # rename temp file
+                    os.rename(temp_save_path, save_path)
+                else:
+                    # remove temp file
+                    os.remove(temp_save_path)
         # model & report to class variables
-        self.model = tagger
-        self.report = report
-        return report, save_path
+        return best_report, save_path
 
 
     @check_model_loaded
